@@ -1,6 +1,8 @@
 use std::cell::RefCell;
+use std::ffi::OsString;
 use std::ops::Deref;
 use std::rc::{Rc, Weak};
+use std::sync::OnceLock;
 use std::time::{Duration, Instant};
 
 use calloop::generic::{FdWrapper, Generic};
@@ -9,6 +11,7 @@ use calloop::{channel, EventLoop, LoopHandle, RegistrationToken};
 use collections::HashMap;
 use copypasta::x11_clipboard::{Clipboard, Primary, X11ClipboardContext};
 use copypasta::ClipboardProvider;
+use parking_lot::Mutex;
 
 use util::ResultExt;
 use x11rb::connection::{Connection, RequestConnection};
@@ -22,25 +25,29 @@ use x11rb::protocol::{randr, render, xinput, xkb, xproto, Event};
 use x11rb::resource_manager::Database;
 use x11rb::xcb_ffi::XCBConnection;
 use xim::{x11rb::X11rbClient, Client};
+use xim::{AHashMap, AttributeName, ClientHandler, InputStyle};
 use xkbc::x11::ffi::{XKB_X11_MIN_MAJOR_XKB_VERSION, XKB_X11_MIN_MINOR_XKB_VERSION};
 use xkbcommon::xkb as xkbc;
 
 use crate::platform::linux::LinuxClient;
-use crate::platform::{LinuxCommon, PlatformWindow};
+use crate::platform::{LinuxCommon, PlatformWindow, WaylandClientState};
 use crate::{
     modifiers_from_xinput_info, point, px, AnyWindowHandle, Bounds, CursorStyle, DisplayId,
-    Modifiers, ModifiersChangedEvent, Pixels, PlatformDisplay, PlatformInput, Point, ScrollDelta,
-    Size, TouchPhase, WindowParams, X11Window,
+    ForegroundExecutor, Keystroke, Modifiers, ModifiersChangedEvent, Pixels, PlatformDisplay,
+    PlatformInput, Point, ScrollDelta, Size, TouchPhase, WindowAppearance, WindowParams, X11Window,
 };
 
 use super::{
     super::{open_uri_internal, SCROLL_LINES},
     X11Display, X11WindowStatePtr, XcbAtoms,
 };
-use super::{button_from_mask, button_of_key, modifiers_from_state};
+use super::{button_of_key, modifiers_from_state, pressed_button_from_mask};
 use super::{XimCallbackEvent, XimHandler};
 use crate::platform::linux::is_within_click_distance;
 use crate::platform::linux::platform::DOUBLE_CLICK_INTERVAL;
+use crate::platform::linux::xdg_desktop_portal::{Event as XDPEvent, XDPEventSource};
+
+pub(super) const XINPUT_MASTER_DEVICE: u16 = 1;
 
 pub(crate) struct WindowRef {
     window: X11WindowStatePtr,
@@ -105,6 +112,9 @@ pub struct X11ClientState {
     pub(crate) ximc: Option<X11rbClient<Rc<XCBConnection>>>,
     pub(crate) xim_handler: Option<XimHandler>,
 
+    pub(crate) compose_state: xkbc::compose::State,
+    pub(crate) pre_edit_text: Option<String>,
+    pub(crate) composing: bool,
     pub(crate) cursor_handle: cursor::Handle,
     pub(crate) cursor_styles: HashMap<xproto::Window, CursorStyle>,
     pub(crate) cursor_cache: HashMap<CursorStyle, xproto::Cursor>,
@@ -180,7 +190,7 @@ impl X11Client {
         );
 
         let master_device_query = xcb_connection
-            .xinput_xi_query_device(1_u16)
+            .xinput_xi_query_device(XINPUT_MASTER_DEVICE)
             .unwrap()
             .reply()
             .unwrap();
@@ -215,8 +225,8 @@ impl X11Client {
             .unwrap();
         assert!(xkb.supported);
 
+        let xkb_context = xkbc::Context::new(xkbc::CONTEXT_NO_FLAGS);
         let xkb_state = {
-            let xkb_context = xkbc::Context::new(xkbc::CONTEXT_NO_FLAGS);
             let xkb_device_id = xkbc::x11::get_core_keyboard_device_id(&xcb_connection);
             let xkb_keymap = xkbc::x11::keymap_new_from_device(
                 &xkb_context,
@@ -226,6 +236,17 @@ impl X11Client {
             );
             xkbc::x11::state_new_from_device(&xkb_keymap, &xcb_connection, xkb_device_id)
         };
+        let compose_state = {
+            let locale = std::env::var_os("LC_CTYPE").unwrap_or(OsString::from("C"));
+            let table = xkbc::compose::Table::new_from_locale(
+                &xkb_context,
+                &locale,
+                xkbc::compose::COMPILE_NO_FLAGS,
+            )
+            .log_err()
+            .unwrap();
+            xkbc::compose::State::new(&table, xkbc::compose::STATE_NO_FLAGS)
+        };
 
         let screen = xcb_connection.setup().roots.get(x_root_index).unwrap();
 
@@ -342,6 +363,17 @@ impl X11Client {
                 }
             })
             .expect("Failed to initialize XIM event source");
+        handle.insert_source(XDPEventSource::new(&common.background_executor), {
+            move |event, _, client| match event {
+                XDPEvent::WindowAppearance(appearance) => {
+                    client.with_common(|common| common.appearance = appearance);
+                    for (_, window) in &mut client.0.borrow_mut().windows {
+                        window.window.set_appearance(appearance);
+                    }
+                }
+            }
+        });
+
         X11Client(Rc::new(RefCell::new(X11ClientState {
             event_loop: Some(event_loop),
             loop_handle: handle,
@@ -361,6 +393,10 @@ impl X11Client {
             ximc,
             xim_handler,
 
+            compose_state: compose_state,
+            pre_edit_text: None,
+            composing: false,
+
             cursor_handle,
             cursor_styles: HashMap::default(),
             cursor_cache: HashMap::default(),
@@ -374,6 +410,58 @@ impl X11Client {
         })))
     }
 
+    pub fn enable_ime(&self) {
+        let mut state = self.0.borrow_mut();
+        if state.ximc.is_none() {
+            return;
+        }
+
+        let mut ximc = state.ximc.take().unwrap();
+        let mut xim_handler = state.xim_handler.take().unwrap();
+        let mut ic_attributes = ximc
+            .build_ic_attributes()
+            .push(
+                AttributeName::InputStyle,
+                InputStyle::PREEDIT_CALLBACKS
+                    | InputStyle::STATUS_NOTHING
+                    | InputStyle::PREEDIT_NONE,
+            )
+            .push(AttributeName::ClientWindow, xim_handler.window)
+            .push(AttributeName::FocusWindow, xim_handler.window);
+
+        let window_id = state.focused_window;
+        drop(state);
+        if let Some(window_id) = window_id {
+            let window = self.get_window(window_id).unwrap();
+            if let Some(area) = window.get_ime_area() {
+                ic_attributes =
+                    ic_attributes.nested_list(xim::AttributeName::PreeditAttributes, |b| {
+                        b.push(
+                            xim::AttributeName::SpotLocation,
+                            xim::Point {
+                                x: u32::from(area.origin.x + area.size.width) as i16,
+                                y: u32::from(area.origin.y + area.size.height) as i16,
+                            },
+                        );
+                    });
+            }
+        }
+        ximc.create_ic(xim_handler.im_id, ic_attributes.build());
+        state = self.0.borrow_mut();
+        state.xim_handler = Some(xim_handler);
+        state.ximc = Some(ximc);
+    }
+
+    pub fn disable_ime(&self) {
+        let mut state = self.0.borrow_mut();
+        state.composing = false;
+        if let Some(mut ximc) = state.ximc.take() {
+            let xim_handler = state.xim_handler.as_ref().unwrap();
+            ximc.destroy_ic(xim_handler.im_id, xim_handler.ic_id);
+            state.ximc = Some(ximc);
+        }
+    }
+
     fn get_window(&self, win: xproto::Window) -> Option<X11WindowStatePtr> {
         let state = self.0.borrow();
         state
@@ -419,12 +507,21 @@ impl X11Client {
             Event::FocusIn(event) => {
                 let window = self.get_window(event.event)?;
                 window.set_focused(true);
-                self.0.borrow_mut().focused_window = Some(event.event);
+                let mut state = self.0.borrow_mut();
+                state.focused_window = Some(event.event);
+                drop(state);
+                self.enable_ime();
             }
             Event::FocusOut(event) => {
                 let window = self.get_window(event.event)?;
                 window.set_focused(false);
-                self.0.borrow_mut().focused_window = None;
+                let mut state = self.0.borrow_mut();
+                state.focused_window = None;
+                state.compose_state.reset();
+                state.pre_edit_text.take();
+                drop(state);
+                self.disable_ime();
+                window.handle_ime_delete();
             }
             Event::XkbStateNotify(event) => {
                 let mut state = self.0.borrow_mut();
@@ -452,12 +549,44 @@ impl X11Client {
                 let modifiers = modifiers_from_state(event.state);
                 let keystroke = {
                     let code = event.detail.into();
-                    let keystroke = crate::Keystroke::from_xkb(&state.xkb, modifiers, code);
+                    let mut keystroke = crate::Keystroke::from_xkb(&state.xkb, modifiers, code);
                     state.xkb.update_key(code, xkbc::KeyDirection::Down);
                     let keysym = state.xkb.key_get_one_sym(code);
                     if keysym.is_modifier_key() {
                         return Some(());
                     }
+                    state.compose_state.feed(keysym);
+                    match state.compose_state.status() {
+                        xkbc::Status::Composed => {
+                            state.pre_edit_text.take();
+                            keystroke.ime_key = state.compose_state.utf8();
+                            keystroke.key =
+                                xkbc::keysym_get_name(state.compose_state.keysym().unwrap());
+                        }
+                        xkbc::Status::Composing => {
+                            state.pre_edit_text = state
+                                .compose_state
+                                .utf8()
+                                .or(crate::Keystroke::underlying_dead_key(keysym));
+                            let pre_edit = state.pre_edit_text.clone().unwrap_or(String::default());
+                            drop(state);
+                            window.handle_ime_preedit(pre_edit);
+                            state = self.0.borrow_mut();
+                        }
+                        xkbc::Status::Cancelled => {
+                            let pre_edit = state.pre_edit_text.take();
+                            drop(state);
+                            if let Some(pre_edit) = pre_edit {
+                                window.handle_ime_commit(pre_edit);
+                            }
+                            if let Some(current_key) = Keystroke::underlying_dead_key(keysym) {
+                                window.handle_ime_preedit(current_key);
+                            }
+                            state = self.0.borrow_mut();
+                            state.compose_state.feed(keysym);
+                        }
+                        _ => {}
+                    }
                     keystroke
                 };
                 drop(state);
@@ -493,6 +622,19 @@ impl X11Client {
                     px(event.event_x as f32 / u16::MAX as f32 / state.scale_factor),
                     px(event.event_y as f32 / u16::MAX as f32 / state.scale_factor),
                 );
+
+                if state.composing && state.ximc.is_some() {
+                    drop(state);
+                    self.disable_ime();
+                    self.enable_ime();
+                    window.handle_ime_unmark();
+                    state = self.0.borrow_mut();
+                } else if let Some(text) = state.pre_edit_text.take() {
+                    state.compose_state.reset();
+                    drop(state);
+                    window.handle_ime_commit(text);
+                    state = self.0.borrow_mut();
+                }
                 if let Some(button) = button_of_key(event.detail.try_into().unwrap()) {
                     let click_elapsed = state.last_click.elapsed();
 
@@ -542,7 +684,7 @@ impl X11Client {
             Event::XinputMotion(event) => {
                 let window = self.get_window(event.event)?;
                 let state = self.0.borrow();
-                let pressed_button = button_from_mask(event.button_mask[0]);
+                let pressed_button = pressed_button_from_mask(event.button_mask[0]);
                 let position = point(
                     px(event.event_x as f32 / u16::MAX as f32 / state.scale_factor),
                     px(event.event_y as f32 / u16::MAX as f32 / state.scale_factor),
@@ -625,7 +767,7 @@ impl X11Client {
 
                 let window = self.get_window(event.event)?;
                 let state = self.0.borrow();
-                let pressed_button = button_from_mask(event.buttons[0]);
+                let pressed_button = pressed_button_from_mask(event.buttons[0]);
                 let position = point(
                     px(event.event_x as f32 / u16::MAX as f32 / state.scale_factor),
                     px(event.event_y as f32 / u16::MAX as f32 / state.scale_factor),
@@ -674,6 +816,9 @@ impl X11Client {
 
     fn xim_handle_commit(&self, window: xproto::Window, text: String) -> Option<()> {
         let window = self.get_window(window).unwrap();
+        let mut state = self.0.borrow_mut();
+        state.composing = false;
+        drop(state);
 
         window.handle_ime_commit(text);
         Some(())
@@ -686,6 +831,7 @@ impl X11Client {
         let mut state = self.0.borrow_mut();
         let mut ximc = state.ximc.take().unwrap();
         let mut xim_handler = state.xim_handler.take().unwrap();
+        state.composing = true;
         drop(state);
 
         if let Some(area) = window.get_ime_area() {
@@ -773,6 +919,7 @@ impl LinuxClient for X11Client {
             x_window,
             &state.atoms,
             state.scale_factor,
+            state.common.appearance,
         );
 
         let screen_resources = state
@@ -826,7 +973,7 @@ impl LinuxClient for X11Client {
                         .unwrap();
                     let _ = state.xcb_connection.flush().unwrap();
                     // Take into account that some frames have been skipped
-                    let now = time::Instant::now();
+                    let now = Instant::now();
                     while instant < now {
                         instant += refresh_duration;
                     }
